package com.antoniotari.reactiveampacheapp.utils;

import android.content.Context;
import android.support.v4.util.ArrayMap;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Map;
import java.util.concurrent.Executor;
import java.util.concurrent.Executors;

import com.antoniotari.reactiveampache.models.Album;
import com.antoniotari.reactiveampache.models.Artist;
import com.antoniotari.reactiveampache.models.Playlist;
import com.antoniotari.reactiveampache.models.Song;
import com.antoniotari.reactiveampache.utils.Log;

import okhttp3.OkHttpClient;
import okhttp3.Request;
import okhttp3.Response;
import rx.Observable;
import rx.Observable.OnSubscribe;
import rx.android.schedulers.AndroidSchedulers;
import rx.schedulers.Schedulers;

/**
 * Created by antoniotari on 2017-01-09.
 */

public enum AmpacheCache {
    INSTANCE;

    private OkHttpClient mOkHttpClient;

    public static final Executor SERIAL_EXECUTOR = Executors.newSingleThreadExecutor();

    private final Map<String,Artist> mArtistMap = new ArrayMap<>();
    private final Map<String,Album> mAlbumMap = new ArrayMap<>();
    private final Map<String,Song> mSongMap = new ArrayMap<>();
    private final Map<String,Playlist> mPlaylistMap = new ArrayMap<>();

    public void addArtistToCache(Artist artist) {
        mArtistMap.put(artist.getId(),artist);
    }

    public void addAlbumToCache(Album album) {
        mAlbumMap.put(album.getId(),album);
    }

    public void addPlaylistToCache(Playlist playlist) {
        mPlaylistMap.put(playlist.getName(), playlist);
    }

    public void addSongToCache(Song song) {
        mSongMap.put(song.getId(),song);
    }

    public Artist getArtist(String id) {
        return mArtistMap.get(id);
    }

    public Album getAlbum(String id) {
        return mAlbumMap.get(id);
    }

    public Song getSong(String id) {
        return mSongMap.get(id);
    }

    public Playlist getPlaylist(final String playlistName) {
        return mPlaylistMap.get(playlistName);
    }

    private OkHttpClient getOkHttpClient() {
        if (mOkHttpClient == null) {
            mOkHttpClient = new OkHttpClient();
        }
        return mOkHttpClient;
    }

    public Observable<Song> downloadSong(Context context, final Song song) {
        final File file = new File(context.getCacheDir(), song.getId());
        return Observable.create((OnSubscribe<Song>) subscriber -> {
            try {
                Log.blu("before", song.getTitle());
                copyInputStreamToFile(downloadBinaryFile(song.getUrl()), file);
                Log.blu("after", song.getTitle());
                subscriber.onNext(song);
                subscriber.onCompleted();
            } catch (Exception e) {
                subscriber.onError(e);
            }
        }).subscribeOn(Schedulers.from(SERIAL_EXECUTOR)).observeOn(AndroidSchedulers.mainThread());
    }

    private InputStream downloadBinaryFile(final String url) throws IOException {
        Request request = new Request.Builder().url(url).build();
        Response response = getOkHttpClient().newCall(request).execute();
        if (!response.isSuccessful()) {
            throw new IOException("Unexpected code " + response);
        }
        return response.body().byteStream();
    }

    private void copyInputStreamToFile(InputStream in, File file) throws IOException {
        OutputStream out = new FileOutputStream(file);
        byte[] buf = new byte[1024];
        int len;
        while ((len = in.read(buf)) > 0) {
            out.write(buf, 0, len);
        }
        out.close();
        in.close();
    }

    public boolean isSongInCache(Context context, Song song) {
        File file = new File(context.getCacheDir(), song.getId());
        return (file.exists());
    }

    public String getCachedSongPath(Context context, Song song) {
        File file = new File(context.getCacheDir(), song.getId());
        if (file.exists()) {
            return file.getPath();
        }
        return null;
    }
}
